<?php

namespace TVn7\UserBundle\Command;


use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Input\InputOption;
use TVn7\UserBundle\Entity\Membre;
use Symfony\Component\Security\Core\Role\Role;
use TVn7\LdapBundle\Entity\User;
use Symfony\Component\Console\Style\SymfonyStyle;

class SyncLDAPMemberCommand extends ContainerAwareCommand {


    protected function configure() {
        $this
            ->setName('TVn7:ldap:sync')
            ->setDescription('Sync ldap members with TDB');

    }

    protected function execute(InputInterface $input, OutputInterface $output) {
        $io = new SymfonyStyle($input, $output); // https://symfony.com/doc/current/console/style.html
        if($input->getOption('verbose')) {
            $io->title('Sync ldap members with TDB');
        }
        $em = $this->getContainer()->get('doctrine')->getManager();

        $membres = $em->getRepository('TVn7UserBundle:Membre')->findAll();

        $ldap = $this->getContainer()->get('tvn7.ldap.driver');
        $sync = $this->getContainer()->get('tvn7.user.ldap.sync');
        $logger = $this->getContainer()->get('monolog.logger.security');

        if($input->getOption('verbose')) $io->section('TDB\'s Members checks');

        foreach($membres as $membre) {
            if($input->getOption('verbose')) {
                $io->note('Checking "' . $membre->getUsername() . '"');
            }
            $result = $sync->checkGroups($membre);
            
            foreach($result['add'] as $group) {
                $io->warning($membre->getUsername() . ' added to ' . $group);
            }
            foreach($result['del'] as $group) {
                $io->warning($membre->getUsername() . ' removed from ' . $group);
            }
        }

        if($input->getOption('verbose')) $io->section('Other Members checks');

        $usernameList = array_map(function($m) {
            return $m->getUsername();
        }, $membres);
        
        foreach($sync->getLDAPGroups() as $group) {
            if($input->getOption('verbose')) $io->note('Checking members in group ' . $group['alias']);
            $groupMembers = $ldap->getGroupMembers($group['dn']);
            foreach($groupMembers as $user) {
                if(!in_array($user, $usernameList)) {
                    $ldapUser = $ldap->getUserProfileByUid($user);
                    if ($ldapUser instanceof User) {
                        $msg = "User '" . $ldapUser->getCn() . " (" . $user . ")' in LDAP group '".$group['alias']."' but not registered on TDB. Removing...";
                        $logger->addWarning($msg);
                        $io->warning($msg);
                    } else {
                        $msg = "User '" . $user . "' in LDAP group '".$group['alias']."' but not registered on LDAP and on TDB. Removing...";
                        $logger->addWarning($msg);
                        $io->warning($msg);
                    }
                    $ldap->remove($group['dn'], $user);
                }
            }
        }        
    }
} 